# Abstraction to allow for both podman and docker to run a container
# For docker, it leverages the docker module's existing `docker::run`
# For podman, it creates start and stop bash scripts and systemd service 
# file to run them to replicate what the docker module does for docker.
#
# @param runtime [Complyadm::Runtime] The runtime to use to run the container
# @param image [String] The image to use as the base for the container
# @param net [Variant[String,Array[String[1],1],Undef]] The existing runtime network to connect to
# @param ports [Variant[String,Array,Undef]] A list of TCP ports to publish in the container
# @param volumes [Variant[String,Array,Undef]] A list of volumes to mount in the container
# @param env [Variant[String,Array]] A list of environment variables to set in the container
# @param env_file [Variant[String,Array]] A list of environment files to set in the container
# @param pull_on_start [Boolean] Whether to pull the image on start.  Not implemented for podman
# @param extra_parameters [Variant[String,Array[String],Undef]] Extra parameters to pass to the runtime
# @param before_start [Variant[String,Boolean]] A command to run before starting the container
# @param before_stop [Variant[String,Boolean]] A command to run before stopping the container
# @param cmd [String] The command to run in the container
# @param after [Variant[String,Array]] A list of containers to start before starting this container
# @param extra_systemd_parameters [Variant[String,Hash]] Extra parameters to pass to the systemd service
# @param after_create [String] Command to run after creating, but before starting container
# @param install_runtime [Boolean] Flag to denote if using a managed runtime environment
define complyadm::runtime::run (
  Complyadm::Runtime $runtime,
  String $image,
  Variant[String,Array[String[1],1],Undef] $net = undef,
  Variant[String,Array,Undef] $ports = [],
  Variant[String,Array,Undef] $volumes = [],
  Variant[String,Array] $env = [],
  Variant[String,Array] $env_file = [],
  Boolean $pull_on_start = false,
  Variant[String,Array[String],Undef] $extra_parameters = undef,
  Variant[String,Boolean] $before_start = false,
  Variant[String,Boolean] $before_stop = false,
  Optional[String] $cmd = undef,
  Variant[String,Array] $after = [],
  Variant[String,Hash] $extra_systemd_parameters = {},
  Optional[String] $after_create = undef,
  Optional[Boolean] $install_runtime = undef,

) {
  if($runtime == 'docker') {
    if($install_runtime == true) {
      include docker
    }
    docker::run { $name:
      image                    => $image,
      net                      => $net,
      ports                    => $ports,
      volumes                  => $volumes,
      env                      => $env,
      env_file                 => $env_file,
      pull_on_start            => $pull_on_start,
      extra_parameters         => $extra_parameters,
      before_start             => $before_start,
      before_stop              => $before_stop,
      command                  => $cmd,
      after                    => $after,
      extra_systemd_parameters => $extra_systemd_parameters,
      after_create             => $after_create,
    }
  } elsif($runtime == 'podman') {
    $systemd_file = "/etc/systemd/system/podman-${name}.service"
    $start_script = "/usr/local/bin/podman-run-${name}-start.sh"
    $stop_script = "/usr/local/bin/podman-run-${name}-stop.sh"
    # create systemd service file
    file { $systemd_file:
      ensure    => file,
      owner     => 'root',
      group     => 'root',
      mode      => '0644',
      show_diff => false,
      content   => epp('complyadm/runtime/service.epp',
        {
          'title'                    => $name,
          'extra_systemd_parameters' => $extra_systemd_parameters,
      }),
    }

    # podman prepends "localhost" to the bitnami image name
    # so we need to update the image name to account for it
    if($image =~ /bitnami/) {
      $image_sanitized = "localhost/${image}"
    } else {
      $image_sanitized = $image
    }

    file { $start_script:
      ensure    => file,
      owner     => 'root',
      group     => 'root',
      mode      => '0770',
      show_diff => false,
      content   => epp('complyadm/runtime/service-start-script.epp',
        {
          'title'            => $name,
          'net'              => $net,
          'envs'             => $env,
          'env_files'        => $env_file,
          'ports'            => $ports,
          'volumes'          => $volumes,
          'extra_parameters' => $extra_parameters,
          'image'            => $image,
          'command'          => $cmd,
          'before_start'     => $before_start,
          'after_create'     => $after_create,
      }),
    }

    file { $stop_script:
      ensure    => file,
      owner     => 'root',
      group     => 'root',
      mode      => '0770',
      show_diff => false,
      content   => epp('complyadm/runtime/service-stop-script.epp',
        {
          'title'       => $name,
          'before_stop' => $before_stop,
      }),
    }

    service { "podman-${name}":
      ensure    => running,
      provider  => 'systemd',
      hasstatus => true,
      require   => [
        File[$systemd_file],
        File[$start_script],
        File[$stop_script],
      ],
      subscribe => [
        File[$systemd_file],
        File[$start_script],
        File[$stop_script],
      ],
    }

    exec { "podman-${name}-systemd-reload": #lint:ignore:check_unsafe_interpolations
      path        => ['/bin/', '/sbin/', '/usr/bin/', '/usr/sbin/'],
      command     => 'systemctl daemon-reload',
      refreshonly => true,
      require     => [
        File[$systemd_file],
        File[$start_script],
        File[$stop_script],
      ],
      subscribe   => [
        File[$systemd_file],
        File[$start_script],
        File[$stop_script],
      ],
    }

    exec { "podman-${name}-systemd-enable": #lint:ignore:check_unsafe_interpolations
      path    => ['/bin/', '/sbin/', '/usr/bin/', '/usr/sbin/'],
      command => "systemctl enable podman-${name} 2>/dev/null",
      unless  => "systemctl status podman-${name} | grep enabled",
      require => Exec["podman-${name}-systemd-reload"],
    }

    Exec["podman-${name}-systemd-reload"] -> Service <| title == "podman-${name}" |> [File[$systemd_file], File[$start_script], File[$stop_script],] ~> Service <| title == "podman-${name}" |>
  } else {
    fail("Unsupported runtime: ${runtime}")
  }
}
