# @api private
#
# @summary Configure MTLS Certs for secure interaction between Puppet Enterprise and Comply
#
# @param [Optional[String]] pe_target_name Run plan non interactively by including Puppet Enterprise
#   within the inventory.yaml and specifying its target name here.
# @param [Optional[String]] hiera_data_file_path Determines where the Comply config is written.
# @param [Optional[String]] pkcs7_public_key_path Path to the public key used to encrypt Hiera data with eyaml.
plan complyadm::configure_pe_certs(
  String $pe_target_name = '',
  String $hiera_data_file_path = 'data/common.yaml',
  String $pkcs7_public_key_path = 'keys/public_key.pkcs7.pem',
) {
  if $pe_target_name != '' {
    # Non Interactive Mode, PE has been provided, create and store certs
    $resolvable_host_name = complyadm::get_yaml_key_value('resolvable_hostname', $hiera_data_file_path)
    $certs = run_plan('complyadm::create_pe_certs', pe_target_name => $pe_target_name, resolvable_host_name => $resolvable_host_name)
    return complyadm::save_pe_certs($certs[0], $certs[1], $certs[2], $hiera_data_file_path, $pkcs7_public_key_path)
  }

  complyadm::display("
A mutual TLS (mTLS) certificate is required for your Puppet Enterprise host to interact with
Security Compliance Management. A mTLS certificate can be automatically generated (provided
the Puppet Enterprise host exists as a target in your Bolt inventory) or provided manually.
mTLS can be configured post-install."
  )

  complyadm::display()
  if Boolean(prompt('Would you like to configure mTLS?', 'default' => 'n')) == false {
    return false
  }

  complyadm::display()
  $inventory_targets = complyadm::bolt_project_inventory_targets()
  if $inventory_targets.size > 1 {
    if Boolean(
      prompt('Automatically generate mTLS certificate?', 'default' => 'n')
    ) {
      complyadm::display()
      $pe_target = prompt::menu('Select Puppet Enterprise target:', $inventory_targets)
      $pe_name = $pe_target.name
      $resolvable_host_name = complyadm::get_yaml_key_value('resolvable_hostname', $hiera_data_file_path)
      $certs = run_plan('complyadm::create_pe_certs', pe_target_name => $pe_name, resolvable_host_name => $resolvable_host_name)
      complyadm::save_pe_certs($certs[0], $certs[1], $certs[2], $hiera_data_file_path, $pkcs7_public_key_path)
      return true
    }
  }

  complyadm::display()
  if Boolean(prompt('Would you like to provide mTLS certificate files?', 'default' => 'n')) == false {
    return false
  }

  complyadm::display()
  if Boolean(prompt('Update TLS Cert?', 'default' => 'n')) {
    $tls_crt = prompt('TLS Cert (Absolute path or Puppet file path)', 'default' => 'complyadm/pe_certs/tls.crt')
    if file::exists($tls_crt) {
      $tls_crt_contents = file::read($tls_crt)
      $tls_crt_updated = complyadm::save_pe_certs($tls_crt_contents, '', '', $hiera_data_file_path, $pkcs7_public_key_path)
    } else {
      out::message("Path ${tls_crt} does not exist, TLS Certificate not updated")
      $tls_crt_updated = false
    }
  } else {
    $tls_crt_updated = false
  }

  complyadm::display()
  if Boolean(prompt('Update TLS Key?', 'default' => 'n')) {
    $tls_key = prompt('TLS Key (Absolute path or Puppet file path)', 'default' => 'complyadm/pe_certs/tls.key')
    if file::exists($tls_key) {
      $tls_key_contents = file::read($tls_key)
      $tls_key_updated = complyadm::save_pe_certs('', $tls_key_contents, '', $hiera_data_file_path, $pkcs7_public_key_path)
    } else {
      out::message("Path ${tls_key} does not exist, TLS Key not updated")
      $tls_key_updated = false
    }
  } else {
    $tls_key_updated = false
  }

  complyadm::display()
  if Boolean(prompt('Update CA Cert?', 'default' => 'n')) {
    $ca_crt = prompt('CA Cert (Absolute path or Puppet file path)', 'default' => 'complyadm/pe_certs/ca.crt')
    if file::exists($ca_crt) {
      $ca_crt_contents = file::read($ca_crt)
      $ca_crt_updated = complyadm::save_pe_certs('', '', $ca_crt_contents, $hiera_data_file_path, $pkcs7_public_key_path)
    } else {
      out::message("Path ${ca_crt} does not exist, CA Certificate not updated")
      $ca_crt_updated = false
    }
  } else {
    $ca_crt_updated = false
  }
  # return true if any changes have made made
  return true in [$tls_crt_updated, $tls_key_updated, $ca_crt_updated]
}
