# @summary Restore Security Compliance Management from a backup created by complyadm::backup.
#
# Use this plan to restore Security Compliance Management from a backup you created with complyadm::backup. To use this plan you need to
# install a fresh Security Compliance Management installation that matches the version of the backup. The database and docker volumes are
# restored from your backup ZIP archive to the fresh installation. You can see a list of the Security Compliance Management backups using
# complyadm::list_backups.
#
# @param backup The name of the backup created by complyadm::backup.  For example "comply-backup-2023-04-05-01-01-01-01.zip"
#
plan complyadm::restore(
  String[1] $backup,
) {
  run_plan('complyadm::check_bolt_version')

  $config = complyadm::config()
  $target = $config['roles']['backend']['targets'][0]
  $runtime = $config['runtime']

  without_default_logging() || {
    $preflight_results = run_plan(
      'complyadm::restore::preflight',
      'config' => $config,
      'backup' => $backup,
    )

    if($preflight_results != '') {
      fail_plan($preflight_results)
    }
  }

  # stop the services
  apply_prep($target, { '_run_as' => 'root' })
  $stop_apply_options = {
    '_run_as' => 'root',
    '_description' => 'Stop Comply Services',
  }

  apply($target, $stop_apply_options) {
    service { "${runtime}-comply_frontdoor":
      ensure => stopped,
    }
    service { "${runtime}-comply_gatekeeper":
      ensure => stopped,
    }
    service { "${runtime}-comply_graphql":
      ensure => stopped,
    }
    service { "${runtime}-comply_identity":
      ensure => stopped,
    }
    service { "${runtime}-comply_mtls_proxy":
      ensure => stopped,
    }
    service { "${runtime}-comply_redis":
      ensure => stopped,
    }
    service { "${runtime}-comply_scarpy":
      ensure => stopped,
    }
    service { "${runtime}-comply_ui":
      ensure => stopped,
    }
  }

  # Gather database information and run the restore task
  $db_role = $config['roles']['database']['services']['comply_postgres']
  $database_info = Complyadm::Support_bundle::Database_info.new({
      'container_name' => $db_role['container']['name'],
      'database_user'  => $db_role['admin_db_username'],
  })

  $host = $config['roles']['backend']['targets'][0]
  $restore_result = run_task(
    'complyadm::restore',
    # TODO: Currently only supports a single target
    $host,
    {
      'runtime'        => $runtime,
      'backup_dir'     => $config['backup_dir'],
      'backup_archive' => $backup,
      'database_info'  => $database_info,
      'database_image' => $config['images']['comply_postgres'],
      'assessor_image' => $config['images']['comply_scarpy_assessor_init'],
      '_run_as'        => 'root',
      '_catch_errors'  => true,
    }
  )

  if($restore_result[0].ok) {
    $restore_result[0].value['warnings'].each |$warning| {
      out::message($warning)
    }
    out::message($restore_result[0].value['message'])
  } else {
    $error_message = @("ERROR")
      Restore failed:
        ${restore_result[0].value['message']}
        ${restore_result[0].value['error']}
      Check the bolt-debug.log for additional details.
      | ERROR
    fail_plan($error_message)
  }

  # Restart the services
  apply_prep($target, { '_run_as' => 'root' })
  $start_apply_options = {
    '_run_as' => 'root',
    '_description' => 'Start Comply Services',
  }

  apply($target, $start_apply_options) {
    service { "${runtime}-comply_frontdoor":
      ensure => running,
    }
    service { "${runtime}-comply_gatekeeper":
      ensure => running,
    }
    service { "${runtime}-comply_graphql":
      ensure => running,
    }
    service { "${runtime}-comply_identity":
      ensure => running,
    }
    service { "${runtime}-comply_mtls_proxy":
      ensure => running,
    }
    service { "${runtime}-comply_postgres":
      ensure => running,
    }
    service { "${runtime}-comply_redis":
      ensure => running,
    }
    service { "${runtime}-comply_scarpy":
      ensure => running,
    }
    service { "${runtime}-comply_ui":
      ensure => running,
    }
  }

  out::message('Restore complete. Note that if the database passwords have been changed, you will need to run')
  out::message('bolt plan run complyadm::configure to update the passwords and allow the services to properly start.')
}
