#!/opt/puppetlabs/puppet/bin/ruby
require 'json'
require 'tmpdir'
require 'open3'
require 'fileutils'
# This task exists to minimize the number of SSH connections used by the collection of data from a Comply installation

# @param [String] tmpdir Destination for collected system info
def collect_system_info(tmpdir)
  system_dir = File.join(tmpdir, 'system')
  Dir.mkdir(system_dir)
  commands = [
    { 'name' => 'top', 'command' => 'top -bn 5' },
    { 'name' => 'vmstat', 'command' => 'vmstat 1 5' },
    { 'name' => 'iostat', 'command' => 'iostat 1 5' },
    { 'name' => 'meminfo', 'command' => 'cat /proc/meminfo' },
    { 'name' => 'cpuinfo', 'command' => 'cat /proc/cpuinfo' },
    { 'name' => 'df_h', 'command' => 'df --human-readable' },
    { 'name' => 'df_i', 'command' => 'df --inodes' },
    { 'name' => 'iptables', 'command' => 'iptables --verbose --list' },
    { 'name' => 'iptables_nat', 'command' => 'iptables --table nat --verbose --list' },
    { 'name' => 'systemctl', 'command' => 'systemctl list-units' },
    { 'name' => 'uptime', 'command' => 'uptime' },
    { 'name' => 'netstat', 'command' => 'netstat --all --numeric --program --tcp --udp --timers' },
    { 'name' => 'dmesg', 'command' => 'dmesg --ctime --time-format iso' },
  ]

  # All calls to capture2e end with semi-colon so that the command forces shell invocation. This is done so the command can
  # exit with code 127 if it doesn't exist. Without the semi-colon an exception is raised. See:
  # https://stackoverflow.com/questions/26040249/why-does-open3-popen3-return-wrong-error-when-executable-is-missing
  commands.each do |command|
    combined_output, = Open3.capture2e("#{command['command']};")
    File.write(File.join(system_dir, "#{command['name']}.txt"), combined_output)
  end
end

# Collect container runtime info
# @param [String] runtime Used to gather information from the container runtime
# @param [String] tmpdir Destination for collected system info
def collect_runtime_info(tmpdir, runtime)
  runtime_commands = [
    { 'name' => 'network_ls', 'command' => "#{runtime} network ls" },
    { 'name' => 'volume_ls', 'command' => "#{runtime} volume ls" },
    { 'name' => 'ps', 'command' => "#{runtime} ps --all" },
    { 'name' => 'journalctl', 'command' => "journalctl --output cat --unit #{runtime}" },
  ]

  runtime_dir = File.join(tmpdir, 'runtime')
  Dir.mkdir(runtime_dir)

  runtime_commands.each do |command|
    combined_output, = Open3.capture2e("#{command['command']};")
    File.write(File.join(runtime_dir, "#{command['name']}.txt"), combined_output)
  end

  containers_dir = File.join(runtime_dir, 'containers')
  Dir.mkdir(containers_dir)
  container_ls_output, = Open3.capture2e("#{runtime} container ls -q")
  container_ls_output.split("\n").each do |container_id|
    combined_output, = Open3.capture2e("#{runtime} container inspect #{container_id};")
    container_info = JSON.parse(combined_output)
    name = container_info[0]['Name']
    redacted_container_info = redact_container_info(container_info)
    File.write(File.join(containers_dir, "#{name}.json"), JSON.pretty_generate(redacted_container_info))
  end

  volumes_dir = File.join(runtime_dir, 'volumes')
  Dir.mkdir(volumes_dir)
  volume_ls_output, = Open3.capture2e("#{runtime} volume ls -q")
  volume_ls_output.split("\n").each do |volume_id|
    combined_output, = Open3.capture2e("#{runtime} volume inspect #{volume_id};")
    volume_info = JSON.parse(combined_output)
    name = volume_info[0]['Name']
    File.write(File.join(volumes_dir, "#{name}.json"), JSON.pretty_generate(volume_info))
  end

  networks_dir = File.join(runtime_dir, 'networks')
  Dir.mkdir(networks_dir)
  network_ls_output, = Open3.capture2e("#{runtime} network ls -q")
  network_ls_output.split("\n").each do |volume_id|
    combined_output, = Open3.capture2e("#{runtime} network inspect #{volume_id};")
    network_info = JSON.parse(combined_output)
    # Podman and docker have different capitalization for the name field
    # specifically in the network inspect output, so we need to check for both
    name = network_info[0].key?('name') ? network_info[0]['name'] : network_info[0]['Name']
    File.write(File.join(networks_dir, "#{name}.json"), JSON.pretty_generate(network_info))
  end
end

# @param [Hash] container_info The output of running a container inspect command.
# Should be the same across all container runtimes.
def redact_container_info(container_info)
  allowed_env_vars_comply_frontdoor = [
    'NGINX_VERSION',
    'NJS_VERSION',
    'PATH',
    'PKG_RELEASE',
  ]

  allowed_env_vars_comply_gatekeeper = [
    'PATH',
  ]

  allowed_env_vars_comply_graphql = [
    # 'HASURA_GRAPHQL_ADMIN_SECRET', # excluded as sensitive
    'HASURA_GRAPHQL_AUTH_HOOK',
    # 'HASURA_GRAPHQL_DATABASE_URL', # excluded as sensitive
    'HASURA_GRAPHQL_ENABLE_CONSOLE',
    'HASURA_GRAPHQL_ENABLE_TELEMETRY',
    'HASURA_GRAPHQL_LOG_LEVEL',
    'HASURA_GRAPHQL_V1_BOOLEAN_NULL_COLLAPSE',
    'HASURA_SCARP_API_URL',
    'HGE_BINARY',
    'LANG',
    'LC_ALL',
    'PATH',
  ]

  allowed_env_vars_comply_identity = [
    'COMPLY_DB_CONNECTION',
    # 'COMPLY_DB_PASSWORD', # excluded as sensitive
    'COMPLY_DB_USER',
    'DB_ADDR',
    # 'DB_PASSWORD', # excluded as sensitive
    'DB_USER',
    'DB_VENDOR',
    'FEATURES',
    'JAVA_OPTS',
    'JBOSS_HOME',
    'JDBC_MARIADB_VERSION',
    'JDBC_MSSQL_VERSION',
    'JDBC_MYSQL_VERSION',
    'JDBC_PARAMS',
    'JDBC_POSTGRES_VERSION',
    'KEYCLOAK_HTTPS_PORT',
    'KEYCLOAK_IMPORT',
    'KEYCLOAK_LOGLEVEL',
    'KEYCLOAK_PASSWORD_FILE',
    'KEYCLOAK_USER_FILE',
    'KEYCLOAK_VERSION',
    'LANG',
    'LAUNCH_JBOSS_IN_BACKGROUND',
    'PATH',
    'PROXY_ADDRESS_FORWARDING',
    'container',
  ]

  allowed_env_vars_comply_postgres = [
    'GOSU_VERSION',
    'LANG',
    'PATH',
    'PGDATA',
    'PG_MAJOR',
    'PG_VERSION',
    # 'POSTGRES_PASSWORD', # excluded as sensitive
  ]

  allowed_env_vars_comply_redis = [
    'GOSU_VERSION',
    'PATH',
    'REDIS_DOWNLOAD_SHA',
    'REDIS_DOWNLOAD_URL',
    'REDIS_VERSION',
  ]

  allowed_env_vars_comply_scarpy = [
    'ASSESSOR_VERSION',
    'CIS_SSL_VERIFY_MODE',
    'COMPLY_PORT',
    'COMPLY_SERVER',
    # 'DB_PASSWORD', # excluded as sensitive
    'DB_USER',
    'DESIRED_SCAN_ENDPOINT',
    'FEATURES',
    # 'HASURA_ADMIN_SECRET', # excluded as sensitive
    'PATH',
    'PE_SKIP_TLS_CHECK',
    'POSTGRES_HOSTNAME',
    'QUERY_API',
    'SCARP_DEBUG_LEVEL',
    'SCARP_ENV',
    'WS_REPORT_PROCESSOR_URL',
  ]

  allowed_env_vars_comply_ui = [
    'ASSESSOR_VERSION',
    'AUTH_PORT',
    'FEATURES',
    'HOST_FQDN',
    'HTTPS_PORT',
    'LOGGING',
    'NODE_VERSION',
    'PATH',
    'QUERY_API',
    'SCARP_API',
    'YARN_VERSION',
  ]

  allowed_env_vars = allowed_env_vars_comply_frontdoor |
    allowed_env_vars_comply_gatekeeper |
    allowed_env_vars_comply_graphql |
    allowed_env_vars_comply_identity |
    allowed_env_vars_comply_postgres |
    allowed_env_vars_comply_redis |
    allowed_env_vars_comply_scarpy |
    allowed_env_vars_comply_ui

  container_info[0]['Config']['Env'].map! do |env|
    # if the string starts with a string not in the allow_list
    # set the string to "*** REDACTED ***"
    # otherwise leave it alone
    env_var_name = env[%r{(.*?)=}, 1]
    unless allowed_env_vars.include?(env_var_name)
      env[%r{=(.*)}, 1] = '[*** REDACTED ***]'
    end
    env
  end
  container_info
end

# Collect log data for each service
# @param [String] tmpdir Destination for collected logs
# @param [String] runtime Used to construct the correct systemd unit name needed to collect logs.
# @param [Array] journald_services A mapping of role to service names of journald logs to collect
def collect_logs(tmpdir, runtime, journald_services)
  log_dir = File.join(tmpdir, 'logs')
  Dir.mkdir(log_dir)
  journald_services.each do |role_services|
    role_log_dir = File.join(log_dir, role_services['role_name'])
    Dir.mkdir(role_log_dir)
    role_services['services'].each do |service|
      service_dir = File.join(role_log_dir, service)
      Dir.mkdir(service_dir)
      combined_output, = Open3.capture2e("journalctl UNIT=#{runtime}-#{service}.service + CONTAINER_NAME=#{service};")
      File.write(File.join(service_dir, "#{service}-journald.log"), combined_output)
    end
  end
end

# Run queries against the Comply database to collect data.
# @param [Hash] database_info Contains info needed to connect to the database.
# @param [String] tmpdir Destination for collected database info.
# @param [String] runtime Used to execute the psql command from within the container.
# @param [String] task_dir The absolute path to the directory of the running Bolt task.
def collect_database_info(database_info, tmpdir, runtime, task_dir)
  db_dir = File.join(tmpdir, 'database')
  Dir.mkdir(db_dir)
  # Collect db table sizes
  File.open(File.join(task_dir, 'complyadm', 'files', 'support_bundle', 'sql', 'table_sizes.sql'), 'r') do |sql_file|
    combined_output, = Open3.capture2e("#{runtime} exec -i #{database_info['container_name']} psql -P pager=off -U #{database_info['database_user']} -f -;", stdin_data: sql_file.read)
    File.write(File.join(db_dir, 'table_sizes.txt'), combined_output)
  end
end

params = JSON.parse(STDIN.read)
tmpdir = Dir.mktmpdir

collect_system_info(tmpdir)
collect_runtime_info(tmpdir, params['runtime'])
collect_logs(tmpdir, params['runtime'], params['journald_services'])
# Collect db info if applicable
unless params['database_info'].nil?
  collect_database_info(params['database_info'], tmpdir, params['runtime'], params['_installdir'])
end

output = { 'tmpdir' => tmpdir }
puts output.to_json
