#!/opt/puppetlabs/puppet/bin/ruby
# Uninstalls Comply
#
# This includes stopping the containers, deleting the associated volumes,
# network, and volumes.  It also deletes the systemd services added during
# the install and removes the Comply directories
#
# The task doesn't stop on failures, as the state of the system is unknown
# we just want to clean up anything that might be there.
#
# @return Array of hashes containing the {command, output} of each command run
require 'open3'
require 'json'
require 'fileutils'

def format_result(result)
  ret = ''
  result.each do |res|
    ret += "#{res['command']}\n#{res['output']}\n"
  end
  ret
end

# Disables the systemd services
#
# @param [Hash] params The parameters passed into the task
# @return Array of hashes of the form:
#   'command' => The command run
#   'output' => The stdout/stderr of the command run
# Disables the systemd services for each container to delete the
# symlinks systemd creates
def disable_systemd_services(params)
  runtime = params['runtime']
  container_delete_results = params['containers'].map do |container|
    runtime_command = "systemctl disable #{runtime}-#{container['name']}"
    output, status = Open3.capture2e(runtime_command)
    {
      'command' => runtime_command,
      'output'  => output,
    }
  end
end

# Deletes the containers and their volumes
#
# @param [Hash] params The parameters passed into the task
# @return Array of hashes of the form:
#   'command' => The command run
#   'output' => The stdout/stderr of the command run
# Inspects each container to get the list of volumes attached.
#
# Uses the provided runtime to delete the application containers
# immediately followed by stopping the systemd services. This needs
# to be done in this order because systemd will automatically restart
# the container when deleted. We do both, deleting the container *and*
# stopping it through systemd to ensure we get rid of as much as possible
# even if one or the other is not fully functional.
#
# Once the containers are deleted, the previously collected volumes are deleted.
def delete_containers_and_volumes(params)
  runtime = params['runtime']
  volumes = []
  container_inspect_results = params['containers'].map do |container|
    runtime_command = "#{runtime} inspect --format json #{container['name']}"
    stdout, status = Open3.capture2(runtime_command)
    inspect_container_results = JSON.parse(stdout)
    unless inspect_container_results.empty?
      inspect_volumes = inspect_container_results[0]['Mounts'].select { |mount| mount.has_key?('Name') }
      inspect_volumes.each { |mount| volumes.push(mount['Name']) }
    end
    {
      'command' => runtime_command,
      'output'  => stdout,
    }
  end
  container_delete_results = params['containers'].map do |container|
    runtime_command =
      "#{runtime} rm --force --volumes #{container['name']} && systemctl stop #{runtime}-#{container['name']}"
    output, status = Open3.capture2e(runtime_command)
    {
      'command' => runtime_command,
      'output'  => output,
    }
  end
  volume_delete_results = volumes.map do |volume|
    runtime_command = "#{runtime} volume rm --force #{volume}"
    output, status = Open3.capture2e(runtime_command)
    {
      'command' => runtime_command,
      'output'  => output,
    }
  end
  container_inspect_results + container_delete_results + volume_delete_results
end

# Delete the Comply images using the given runtime
# Since images from previous versions could be present, we have to look for
# the names of the images from Comply and delete all installed images that match
# the base name
#
# @param [Hash] params The parameters passed into the task
# @return [Array] of hashes of the form:
#   'command' => The command run
#   'output' => The stdout/stderr of the command run
def delete_images(params)
  runtime = params['runtime']

  # Get all of the images installed so we can check for old versions too
  # Capture the image id and name so we can delete by image id, which
  # ensures we get all tags for the image
  #
  # For unknown reasons, podman and docker do not output in the same json
  # format.  Specifically, podman outputs actual json, whereas docker outputs
  # individual lines of json, one for each image.  As a result, we must parse
  # them differently.
  installed_images = {}
  runtime_command = "#{runtime} image ls --format json"
  stdout, status = Open3.capture2(runtime_command)

  if runtime == 'podman'
    JSON.parse(stdout).each do |image|
      next if image['Names'].nil?
      image['Names'].each do |name|
        # podman prepends the image name with localhost/ for images without a registry
        # strip this off so we can match the image names with the config object
        if runtime == 'podman'
          name.sub!(%r{^localhost/}, '')
        end
        installed_images[name] = image['Id']
      end
    end
  else # runtime must be docker
    stdout.lines.each do |image|
      json = JSON.parse(image)
      name = "#{json['Repository']}:#{json['Tag']}"
      if %r{^bitnami/}.match?(name)
        name = "docker.io/#{name}"
      end
      installed_images[name] = json['ID']
    end
  end

  comply_image_names = params['containers'].map { |container| container['image'][%r{(.*?):}, 1] }

  installed_images.select! do |name, _id|
    comply_image_names.include?(name[%r{(.*?):}, 1])
  end

  installed_images.map do |_name, id|
    runtime_command = "#{runtime} image rm --force #{id}"
    output, status = Open3.capture2e(runtime_command)
    {
      'command' => runtime_command,
      'output'  => output,
    }
  end
end

# Deletes the Comply network
#
# @param [Hash] params The parameters passed into the task
# @return [Array] of hashes of the form:
#   'command' => The command run
#   'output' => The stdout/stderr of the command run
# Uses the provided runtime to delete the Comply network
def delete_network(params)
  runtime = params['runtime']
  runtime_command = "#{runtime} network rm --force comply"
  output, status = Open3.capture2e(runtime_command)
  [{
    'command' => runtime_command,
    'output'  => output,
  }]
end

params = JSON.parse(STDIN.read)

# Remove all runtime based items
result = disable_systemd_services(params) +
         delete_containers_and_volumes(params) +
         delete_images(params) +
         delete_network(params)

# Remove the Comply directories
FileUtils.rm_rf('/etc/puppetlabs/comply')
FileUtils.rm_rf('/var/lib/puppetlabs/comply')
FileUtils.rm_rf(params['backup_dir'])

# Remove the systemd init scripts and service files
params['containers'].each do |container|
  FileUtils.rm([
                 "/etc/systemd/system/#{params['runtime']}-#{container['name']}.service",
                 "/usr/local/bin/#{params['runtime']}-run-#{container['name']}-start.sh",
                 "/usr/local/bin/#{params['runtime']}-run-#{container['name']}-stop.sh",
                 "/etc/logrotate.d/comply-#{container['name']}",
               ], force: true)
end

# Return output to caller
output = {
  'message' => format_result(result),
}
puts output.to_json
